@@ -7,18 +7,23 @@
 #' You can also fit binary outcome variables with a probit link function. For that purpose, the response variable has to be defined as "ordered": \code{r1 <- RSA(Z.binary ~ X*Y, dat, ordered="Z.binary")} (for more details see the help file of the \code{sem} function in the \code{lavaan} package.). The results can also be plotted with probabilities on the z axis using the probit link function: \code{plot(r1, link="probit", zlim=c(0, 1), zlab="Probability")}. \code{lavaan} at the moment only supports a probit link function for binary outcomes, not a logit link.
 #'
 #' @export
+#' @import lavaan
+#' @import ggplot2
+#' @import gridExtra
+#' @import lattice
+#' @import RColorBrewer
 #' @param formula A formula in the form \code{z ~ x*y}, specifying the variable names used from the data frame, where z is the name of the response variable, and x and y are the names of the predictor variables.
 #' @param data A data frame with the variables
-#' @param center Should predictor variables be centered on the sample mean before analyses?
-#' @param scale Should predictor variables be scaled to SD = 1 before analyses?
+#' @param center Should predictor variables be centered on \emph{each variable's} sample mean before analyses? You should think carefully about this option, as different centering of the predictor variables can affect the commensurability of the predictor scales.
+#' @param scale Should predictor variables be scales on the SD of \emph{each variable} before analyses? You should think carefully about this option, as different scaling of the predictor variables can affect the commensurability of the predictor scales.
 #' @param na.rm Remove missings before proceeding?
 #' @param out.rm Should outliers according to Bollen & Jackman (1980) criteria be excluded from analyses?
-#' @param breakline Should the breakline in the unconstrained absolute difference model be allowed (the breakline is possible from the model formulation, but empirically rather unrealistic ...)
+#' @param breakline Should the breakline in the unconstrained absolute difference model be allowed (the breakline is possible from the model formulation, but empirically rather unrealistic ...). Defaults to \code{FALSE}
 #' @param verbose Should additional information during the computation process be printed?
-#' @param models A vector with names of all models that should be computed. Should be any from c("absdiff", "absunc", "diff", "additive", "IA", "sqdiff", "RR", "SSD", "SRSD", "full", "null"). For \code{models="all"}, all models are computed, for \code{models="default"} all models besides absolute difference models are computed.
+#' @param models A vector with names of all models that should be computed. Should be any from \code{c("absdiff", "absunc", "diff", "additive", "IA", "sqdiff", "RR", "SSD", "SRSD", "full", "null", "onlyx", "onlyy")}. For \code{models="all"}, all models are computed, for \code{models="default"} all models besides absolute difference models are computed.
 #' @param cubic Should a cubic model with the additional terms Y^3, XY^2, YX^2, and X^3 be included?
 #' @param control.variables A string vector with variable names from \code{data}. These variables are added as linear predictors to the model (in order "to control for them"). No interactions with the other variables are modeled.
-#' @param ... Additional parameters passed to the lavaan sem function. For example: \code{se="boot"}
+#' @param ... Additional parameters passed to the \code{lavaan} \code{\link{sem}} function. For example, you can obtained bootstrapped standard errors by setting \code{se="boot"}.
 #'
 #'
 #' @seealso \code{\link{demoRSA}}, \code{\link{plotRSA}}, \code{\link{RSA.ST}}, \code{\link{confint.RSA}}
@@ -68,14 +73,14 @@ RSA <- function(formula, data=NULL, center=FALSE, scale=FALSE, na.rm=FALSE,
 	out.rm=TRUE, breakline=FALSE, models="default", cubic=FALSE, 
 	verbose=TRUE, control.variables=c(), ...) {
 
-	validmodels <- c("absdiff", "absunc", "diff", "additive", "IA", "sqdiff", "SRRR", "SRR", "RR", "SSD", "SRSD", "full", "null")
+	validmodels <- c("absdiff", "absunc", "diff", "additive", "IA", "sqdiff", "SRRR", "SRR", "RR", "SSD", "SRSD", "full", "null", "onlyx", "onlyy")
 	if (length(models)==1 & models[1]=="all") {models <- validmodels}
-	if (length(models)==1 & models[1]=="default") {models <- c("diff", "additive", "IA", "sqdiff", "SRRR", "SRR", "RR", "SSD", "SRSD", "full", "null")}
+	if (length(models)==1 & models[1]=="default") {models <- c("diff", "additive", "IA", "sqdiff", "SRRR", "SRR", "RR", "SSD", "SRSD", "full", "null", "onlyx", "onlyy")}
 	if (any(!models %in% validmodels))
 		stop("Unknown model name provided in parameter 'models'.")
 	
 	# set all result objects to NULL as default
-	s.NULL <- s.full <- s.IA <- s.diff <- s.absdiff <- s.additive <- s.sqdiff <- s.SSD <- s.SRSD <- s.absunc <- s.cubic <- s.RR <- s.SRR <- s.SRRR <- NULL
+	s.NULL <- s.full <- s.IA <- s.diff <- s.absdiff <- s.additive <- s.sqdiff <- s.SSD <- s.SRSD <- s.absunc <- s.cubic <- s.RR <- s.SRR <- s.SRRR <- s.onlyx <- s.onlyy <- NULL
 	SRSD.rot <- ""
 	SRRR.rot <- ""
 	
@@ -122,16 +127,15 @@ RSA <- function(formula, data=NULL, center=FALSE, scale=FALSE, na.rm=FALSE,
 	f <- paste0(paste0(DV, " ~ ", paste(IV1, IV2, IV12, IV_IA, IV22, sep=" + ")), CV)
 	rs <- lm(f, df)
 	
-	
+	# Mark outliers and influential cases according to Bollen & Jackman, 1980
+	df0 <- df	# df0 keeps the original data frame, before outliers are removed (we need this in plotRSA, if we want to show the outliers)
+	outs <- c()	# keeps the row numbers of the outliers
+	inf <- influence.measures(rs)
+	outs <- which(apply(inf$is.inf[, c("dffit", "cook.d", "hat")], 1, sum) == 3)
+	names(outs) <- NULL
 	if (out.rm == TRUE) {
-		# get outliers and influential cases according to Bollen & Jackman, 1980
-	
-		inf <- influence.measures(rs)
-		outs <- which(apply(inf$is.inf[, c("dffit", "cook.d", "hat")], 1, sum) == 3)
-		if (verbose==TRUE) {
-			print(paste("Removed", length(outs), "case(s) according to Bollen & Jackman (1980) criteria."))
-		}
-		if (length(outs)>0) {
+		if (verbose==TRUE & length(outs)>0) {
+			warning(paste("Removed", length(outs), "case(s) according to Bollen & Jackman (1980) criteria."))
 			df <- df[-outs, ]
 		}
 	}
@@ -162,6 +166,34 @@ withCallingHandlers({
 		sep="\n")
 		s.additive <- sem(m.additive, data=df, fixed.x=TRUE, meanstructure=TRUE, ...)
 	}
+	
+	if ("onlyx" %in% models) {
+		if (verbose==TRUE) print("Computing x + x^2 model (onlyx) ...")
+		m.onlyx <-  paste(poly,
+			"b2==0",
+			"b4==0",
+			"b5==0",
+			"a1 := b1+b2",
+			"a2 := b3+b4+b5",
+			"a3 := b1-b2",
+			"a4 := b3-b4+b5",
+		sep="\n")
+		s.onlyx <- sem(m.onlyx, data=df, fixed.x=TRUE, meanstructure=TRUE, ...)
+	}
+	
+	if ("onlyy" %in% models) {
+		if (verbose==TRUE) print("Computing y + y^2 model (onlyy) ...")
+		m.onlyy <-  paste(poly,
+			"b1==0",
+			"b3==0",
+			"b4==0",
+			"a1 := b1+b2",
+			"a2 := b3+b4+b5",
+			"a3 := b1-b2",
+			"a4 := b3-b4+b5",
+		sep="\n")
+		s.onlyy <- sem(m.onlyy, data=df, fixed.x=TRUE, meanstructure=TRUE, ...)
+	}
 
 	if ("diff" %in% models) {
 		if (verbose==TRUE) print("Computing difference model (diff) ...")
@@ -512,7 +544,7 @@ withCallingHandlers({
 		if (verbose==TRUE) print("Computing unconstrained absolute difference model ...")
 		m.absunc <-  paste(
 			paste0(DV, " ~ b1*", IV1, " + b2*", IV2, " + b6*W + b7*W_", IV1, " + b8*W_", IV2),
-			ifelse(breakline==TRUE, "b6==0", ""),
+			ifelse(breakline==FALSE, "b6==0", ""),
 			sep="\n")
 			
 			s.absunc <- sem(m.absunc, data=df, fixed.x=TRUE, meanstructure=TRUE, ...)
@@ -570,9 +602,9 @@ withCallingHandlers({
 	# ---------------------------------------------------------------------
 	# Build results object
 	res <- list(
-		models = list(null=s.NULL, full=s.full, IA=s.IA, diff=s.diff, absdiff=s.absdiff, additive=s.additive, sqdiff=s.sqdiff, SRRR=s.SRRR, SRR=s.SRR, RR=s.RR, SSD=s.SSD, SRSD=s.SRSD, absunc=s.absunc, cubic=s.cubic), 
+		models = list(null=s.NULL, full=s.full, IA=s.IA, diff=s.diff, absdiff=s.absdiff, additive=s.additive, sqdiff=s.sqdiff, SRRR=s.SRRR, SRR=s.SRR, RR=s.RR, SSD=s.SSD, SRSD=s.SRSD, absunc=s.absunc, cubic=s.cubic, onlyx=s.onlyx, onlyy=s.onlyy), 
 		SRSD.rot = SRSD.rot, SRRR.rot = SRRR.rot, LM=rs, formula=formula, 
-		data=df, DV=DV, IV1=IV1, IV2=IV2, IV12=IV12, IV22=IV22, IV_IA=IV_IA, W_IV1=W_IV1, W_IV2=W_IV2, IV13=IV13, IV23=IV23, IV_IA2=IV_IA2, IV_IA3=IV_IA3, 
+		data=df, data.original=df0, out.rm = out.rm, outliers = outs, DV=DV, IV1=IV1, IV2=IV2, IV12=IV12, IV22=IV22, IV_IA=IV_IA, W_IV1=W_IV1, W_IV2=W_IV2, IV13=IV13, IV23=IV23, IV_IA2=IV_IA2, IV_IA3=IV_IA3, 
 		r.squared = summary(rs)$r.squared)
 	
 	attr(res, "class") <- "RSA"
